@@ -21,6 +21,12 @@
 #ifndef GRD_ENUMS_H
 #define GRD_ENUMS_H
 
+typedef enum
+{
+  GRD_RDP_SCREEN_SHARE_MODE_MIRROR_PRIMARY,
+  GRD_RDP_SCREEN_SHARE_MODE_EXTEND,
+} GrdRdpScreenShareMode;
+
 typedef enum
 {
   GRD_VNC_AUTH_METHOD_PROMPT,
